import cachetest.fwk.TestingUtil;
import java.util.Properties;
import java.util.Timer;
import java.util.TimerTask;

import javax.transaction.Status;
import javax.transaction.TransactionManager;

import org.infinispan.Cache;
import org.infinispan.config.Configuration;
import org.infinispan.config.GlobalConfiguration;
import org.infinispan.manager.DefaultCacheManager;
import org.infinispan.manager.EmbeddedCacheManager;
import org.infinispan.remoting.transport.jgroups.JGroupsTransport;
import org.infinispan.transaction.lookup.JBossStandaloneJTAManagerLookup;
import org.infinispan.transaction.lookup.TransactionManagerLookup;
import org.infinispan.util.concurrent.locks.LockManager;

/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

/**
 *
 * @author yeli
 */
public class CacheScheduledCounter {
    EmbeddedCacheManager manager = null;
   Cache cache = null;
   String key = "locked-counter";

   // Collection<String> keys = Collections.singletonList(key);

   public static void main(String[] args) {
      CacheScheduledCounter c = new CacheScheduledCounter();
      c.start();
   }

   public CacheScheduledCounter() {

      Configuration c = new Configuration();
      c.setCacheMode(Configuration.CacheMode.REPL_SYNC);
      c.setFetchInMemoryState(true);
      c.setUseLockStriping(false);
      
      TransactionManagerLookup txLookup = new JBossStandaloneJTAManagerLookup();
      //TransactionManagerLookup txLookup = new DummyTransactionManagerLookup();
      c.setTransactionManagerLookup(txLookup);

      System.out.println("Create cache manager ...");

      //System.setProperty("jgroups.bind_addr", "localhost");
      //System.setProperty("java.net.preferIPv4Stack", "true");
      GlobalConfiguration gc = GlobalConfiguration.getClusteredDefault();
      amendTransport(gc);
      manager = new DefaultCacheManager(gc,c);
      manager.start();

      System.out.println("get cache");
      cache = manager.getCache("test-grid");
      System.out.println("memebers " + manager.getMembers());
      System.out.println("cache size " + cache.size());
   }



   public void start() {
      Timer timer = new Timer();
      timer.scheduleAtFixedRate(new CounterTask(), 0, 5000);
   }

   class CounterTask extends TimerTask {
      public Object getLockOwner() {
         LockManager lm = TestingUtil.extractComponent(manager.getCache("test-grid"), LockManager.class);
         return lm.getOwner(key);
      }

      @Override
      public void run() {
         TransactionManager tx = cache.getAdvancedCache().getTransactionManager();
         try {
            tx.begin();
         } catch (Exception ex) {
            ex.printStackTrace();
         }

         try {
            System.out.println("aquiring lock on cache " + cache.getName() + " key " + key + "...");
            cache.getAdvancedCache().lock(key);
            System.out.println("lock owner : " + getLockOwner());

            Integer val = (Integer) cache.get(key);
            System.out.println("current value : " + val);
            if (val == null) {
               val = 0;
            } else {
               val++;

            }
            cache.put(key, val);

            try {
                Thread.sleep(2000);
            } catch (Exception ex) {}

            System.out.println("commit...");
            tx.commit();
            System.out.println("done commit");
            System.out.println("lock owner : " + getLockOwner());
         } catch (Exception ex) {
            //ex.printStackTrace();
            try {
               System.out.println("rollback..." + ex.getLocalizedMessage());
               tx.rollback();
               System.out.println("done rollback");
               System.out.println("lock owner : " + getLockOwner());
            } catch (Exception rex) {
               System.out.println("rollback exception..." + rex.getLocalizedMessage());
               System.out.println("lock owner : " + getLockOwner());
            }
         } finally {
            try {
               System.out.print("tx status at the end : ");
               switch (tx.getStatus()) {
                  case Status.STATUS_ACTIVE:
                     System.out.println("active");
                     break;
                  case Status.STATUS_COMMITTED:
                     System.out.println("committed");
                     break;
                  case Status.STATUS_COMMITTING:
                     System.out.println("committing");
                     break;
                  case Status.STATUS_MARKED_ROLLBACK:
                     System.out.println("makerd rollback");
                     break;
                  case Status.STATUS_NO_TRANSACTION:
                     System.out.println("no transaction");
                     break;
                  case Status.STATUS_PREPARED:
                     System.out.println("preprared");
                     break;
                  case Status.STATUS_PREPARING:
                     System.out.println("preparing");
                     break;
                  case Status.STATUS_ROLLEDBACK:
                     System.out.println("rolledback");
                     break;
                  case Status.STATUS_ROLLING_BACK:
                     System.out.println("rolling back");
                     break;
                  case Status.STATUS_UNKNOWN:
                     System.out.println("unknown");
                     break;
                  default:
                     System.out.println(tx.getStatus());
               }
            } catch (Exception ex) {
               ex.printStackTrace();
            }
         }
      }
   }
   private static void amendTransport(GlobalConfiguration configuration) {
      if (configuration.getTransportClass() != null) { //this is local
         Properties newTransportProps = new Properties();
         Properties previousSettings = configuration.getTransportProperties();
         System.out.println("previous settings " + previousSettings);
         if (previousSettings != null) {
            newTransportProps.putAll(previousSettings);
         }
         newTransportProps.put(JGroupsTransport.CONFIGURATION_FILE, "jgroups-tcp.xml");
         configuration.setTransportProperties(newTransportProps);
      }
   }
}
